@@ -62,7 +62,7 @@ character, either by passing and alternative to the C<new> function
 
   my $comp = Array::Compare->new(Sep => '|');
 
-or by changing the seperator for an existing comparator object
+or by changing the separator for an existing comparator object
 
   $comp->Sep('|');
 
@@ -82,9 +82,9 @@ or by altering an existing object:
 
   $comp->WhiteSpace(0);
 
-You can also control whether or not the case of the data is significant 
-in the comparison. The default is that the case of data is taken into 
-account. This can be changed in the standard ways when creating a new 
+You can also control whether or not the case of the data is significant
+in the comparison. The default is that the case of data is taken into
+account. This can be changed in the standard ways when creating a new
 comparator object:
 
   my $comp = Array::Compare->new(Case => 0);
@@ -99,7 +99,7 @@ a full comparison which returns a list containing the indexes of elements
 which differ between the two arrays. If the arrays are the same it returns
 an empty list. In scalar context the full comparison returns the length of
 this list (i.e. the number of elements that differ). You can access the full
-comparision in two ways. Firstly, there is a C<DefFull> attribute. If this
+comparison in two ways. Firstly, there is a C<DefFull> attribute. If this
 is C<true> then a full comparison if carried out whenever the C<compare>
 method is called.
 
@@ -169,30 +169,31 @@ they contain the same elements but in a different order.
     print "Nope. Arrays are completely different\n";
   }
 
-In this case the values of C<WhiteSpace> and C<Case> are still used, 
+In this case the values of C<WhiteSpace> and C<Case> are still used,
 but C<Skip> is ignored for, hopefully, obvious reasons.
 
 =head1 METHODS
 
-=cut 
+=cut
 
 package Array::Compare;
 
-require 5.006_000;
+require 5.010_000;
 use strict;
 use warnings;
 our ($VERSION, $AUTOLOAD);
 
-use Moose;
+use Moo;
+use Types::Standard qw(Str Bool HashRef);
 use Carp;
 
-$VERSION = '2.02';
+$VERSION = '3.0.2';
 
-has Sep        => ( is => 'rw', isa => 'Str',     default => '^G' );
-has WhiteSpace => ( is => 'rw', isa => 'Bool',    default => 1 );
-has Case       => ( is => 'rw', isa => 'Bool',    default => 1 );
-has DefFull    => ( is => 'rw', isa => 'Bool',    default => 0 );
-has Skip       => ( is => 'rw', isa => 'HashRef', default => sub { {} } );
+has Sep        => ( is => 'rw', isa => Str,     default => '^G' );
+has WhiteSpace => ( is => 'rw', isa => Bool,    default => 1 );
+has Case       => ( is => 'rw', isa => Bool,    default => 1 );
+has DefFull    => ( is => 'rw', isa => Bool,    default => 0 );
+has Skip       => ( is => 'rw', isa => HashRef, default => sub { {} } );
 
 =head2 new [ %OPTIONS ]
 
@@ -245,9 +246,14 @@ comparison).
 #
 sub _check_args {
   my $self = shift;
-  croak('Must compare two arrays.') unless @_ == 2;
-  croak('Argument 1 is not an array') unless ref($_[0]) eq 'ARRAY';
-  croak('Argument 2 is not an array') unless ref($_[1]) eq 'ARRAY';
+
+  my @errs;
+
+  push @errs, 'Must compare two arrays.' unless @_ == 2;
+  push @errs, 'Argument 1 is not an array' unless ref($_[0]) eq 'ARRAY';
+  push @errs, 'Argument 2 is not an array' unless ref($_[1]) eq 'ARRAY';
+
+  croak join "\n", @errs if @errs;
 
   return;
 }
@@ -267,6 +273,19 @@ sub compare_len {
   return @{$_[0]} == @{$_[1]};
 }
 
+=head2 different_len \@ARR1, \@ARR2
+
+Passed two arrays and returns true if they are of different lengths.
+
+This is just the inverse of C<compare_len> (which is badly named).
+
+=cut
+
+sub different_len {
+  my $self = shift;
+  return ! $self->compare_len(@_);
+}
+
 =head2 compare \@ARR1, \@ARR2
 
 Compare the values in two arrays and return a data indicating whether
@@ -322,9 +341,8 @@ sub simple_compare {
   # Filter @check so it only contains indexes that should be compared.
   # N.B. Makes no sense to do this if we are called from 'perm'.
   unless ($perm) {
-    @check = grep {!(exists $self->Skip->{$_}
-		     && $self->Skip->{$_}) } @check
-		       if keys %{$self->Skip};
+    @check = grep {!(exists $self->Skip->{$_} && $self->Skip->{$_}) } @check
+      if keys %{$self->Skip};
   }
 
   # Build two strings by taking array slices containing only the columns
@@ -384,18 +402,8 @@ sub full_compare {
   # the number of the columns that appear in the longer list and aren't
   # in the shorter list. If we've been called in scalar context we
   # return the difference in the lengths of the two lists.
-  unless ($self->compare_len(@_)) {
-    if (wantarray) {
-      my ($max, $min);
-      if ($#{$row1} > $#{$row2}) {
-	($max, $min) = ($#{$row1}, $#{$row2} + 1);
-      } else {
-	($max, $min) = ($#{$row2}, $#{$row1} + 1);
-      }
-      return ($min .. $max);
-    } else {
-      return abs(@{$row1} - @{$row2});
-    }
+  if ($self->different_len(@_)) {
+    return $self->_different_len_returns(@_);
   }
 
   my ($arr1, $arr2) = @_;
@@ -407,11 +415,8 @@ sub full_compare {
 
     my ($val1, $val2) = ($arr1->[$_], $arr2->[$_]);
 
-    next unless defined $val1 or defined $val2;
-
-    if ((defined $val1 and not defined $val2)
-      or (defined $val2 and not defined $val1)) {
-      push @diffs, $_;
+    if (not defined $val1 or not defined $val2) {
+      push @diffs, $_ if $self->_defined_diff($val1, $val2);
       next;
     }
 
@@ -431,6 +436,30 @@ sub full_compare {
   return wantarray ? @diffs : scalar @diffs;
 }
 
+sub _different_len_returns {
+  my $self = shift;
+  my ($row1, $row2) = @_;
+
+  if (wantarray) {
+    if ($#{$row1} > $#{$row2}) {
+      return ( $#{$row2} + 1 .. $#{$row1} );
+    } else {
+      return ( $#{$row1} + 1 .. $#{$row2} );
+    }
+  } else {
+    return abs(@{$row1} - @{$row2});
+  }
+}
+
+sub _defined_diff {
+  my $self = shift;
+  my ($val1, $val2) = @_;
+
+  return   if not defined $val1 and not defined $val2;
+  return 1 if     defined $val1 and not defined $val2;
+  return 1 if not defined $val1 and     defined $val2;
+}
+
 =head2 perm \@ARR1, \@ARR2
 
 Check to see if one array is a permutation of the other (i.e. contains
@@ -449,9 +478,6 @@ sub perm {
   return $self->simple_compare([sort @{$_[0]}], [sort @{$_[1]}]);
 }
 
-no Moose;
-__PACKAGE__->meta->make_immutable;
-
 1;
 __END__
 
@@ -468,6 +494,6 @@ perl(1).
 Copyright (C) 2000-2005, Magnum Solutions Ltd.  All Rights Reserved.
 
 This script is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself. 
+under the same terms as Perl itself.
 
 =cut
